use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use crate::compare::ElispFact;
use crate::types::PropertyDrawer;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmPropertyDrawer {}

to_wasm!(
    WasmPropertyDrawer,
    PropertyDrawer<'s>,
    original,
    wasm_context,
    { WasmAstNode::PropertyDrawer(original) },
    { "property-drawer".into() },
    {
        let children = original
            .children
            .iter()
            .map(|child| {
                child
                    .to_wasm(wasm_context.clone())
                    .map(Into::<WasmAstNode>::into)
            })
            .collect::<Result<Vec<_>, _>>()?;

        Ok((children, WasmPropertyDrawer {}))
    }
);
